/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

var accountsTree, accountsDeck, copyCmd, deleteAccountFolderOrItemCmd, settingsPopupMenuItem,
  accountSettingsCmd;

function initAccountsPanel() {

  accountsTree = document.getElementById("accountsTree");
  accountsDeck = document.getElementById("accountsDeck");
  deletePopupMenuItem = document.getElementById("deletePopupMenuItem");
  newFolderPopupMenuItem = document.getElementById("newFolderPopupMenuItem");
  newAccountPopupMenuItem = document.getElementById("newAccountPopupMenuItem");
  otherFieldsPopupMenuItem = document.getElementById("otherFieldsPopupMenuItem");
  settingsPopupMenuItem = document.getElementById("settingsPopupMenuItem");
  copyCmd = document.getElementById("copyCmd");
  deleteAccountFolderOrItemCmd = document.getElementById("deleteAccountFolderOrItemCmd");
  newFolderPopupMenuItem = document.getElementById("newFolderPopupMenuItem");
  accountSettingsCmd = document.getElementById("accountSettingsCmd");
  
  accountsTree.database.AddDataSource(passwordMakerHelper.datasource);
  accountsTree.builder.rebuild();
  accountsTree.view.selection.select(0);

  accountsTree.addEventListener("click", function(event) {
    var row = {}, col = {}, childElt = {};
    this.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, childElt);
    if (event.detail == 2 && childElt.value != "twisty" && row.value > -1) {
      onAccountSettings();
      event.stopPropagation();
    }
  }, true);

  /*accountsTree.addEventListener("contextmenu", function(event) {
    var row = {}, col = {}, childElt = {};
    this.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, childElt);
    switch (row.value) {
      case -1:
        // No tree item selected
        newFolderPopupMenuItem.setAttribute("disabled", false); 
        newAccountPopupMenuItem.setAttribute("disabled", true);             
        deletePopupMenuItem.setAttribute("disabled", true);
        settingsPopupMenuItem.setAttribute("disabled", true);   
        break;
      case 0:
        newFolderPopupMenuItem.setAttribute("disabled", false); 
        newAccountPopupMenuItem.setAttribute("disabled", true);             
        deletePopupMenuItem.setAttribute("disabled", true);
        settingsPopupMenuItem.setAttribute("disabled", false);        
        break;
      default:
        newFolderPopupMenuItem.setAttribute("disabled", false); 
        newAccountPopupMenuItem.setAttribute(
          accountsTree.view.getItemAtIndex(accountsTree.currentIndex).getAttribute("container") == "true",
          false);             
        deletePopupMenuItem.setAttribute("disabled", false);
        settingsPopupMenuItem.setAttribute("disabled", false);       
    }
  }, true);*/
}

function onAccountsTreeSelection() {
  // Disable delete button if there's no selection or the "defaults" row is selected.
  if (accountsTree.currentIndex == -1) {
    // Nothing selected. select the first item
    accountsTree.view.selection.select(0);
    return;
  }
  defaultAccountOptionsSelected = accountsTree.currentIndex == 0;
  deleteAccountFolderOrItemCmd.setAttribute("disabled", defaultAccountOptionsSelected);

  // if a folder...
  if (accountsTree.view.getItemAtIndex(accountsTree.currentIndex).getAttribute("container") == "true") {
    newAccountCmd.setAttribute("disabled", false); // Allow creation of new accounts 
    pwdDeck.selectedIndex="1";
    copyCmd.setAttribute("disabled", true);
  }
  else {
    newAccountCmd.setAttribute("disabled", true); // Disallow creation of new accounts
    pwdDeck.selectedIndex="0";
    updateURLLabel();
    generatepassword();   // If not a folder, generate password
    copyCmd.setAttribute("disabled", false);
  }
}

function promptForTwoItems(nameLabel, descLabel) {
   var params = {inn: {name:nameLabel, description:descLabel}, out:null};

   window.openDialog("chrome://passwdmaker/chrome/treeprompt.xul",
      "", "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK. 
    while (params.out != null && params.out.folderName == "") {
      // MUST enter a folder name
      passwordMakerHelper.prompts.alert(this, "PasswordMaker", "A name is required.");
      params.out = null;
      window.openDialog("chrome://passwdmaker/chrome/treeprompt.xul", "", "chrome, dialog, modal, resizable=yes",
        params).focus();
    }
    return params.out;
  }
}

function createNewFolder(folderNameURI, folderDescURI, containerSeq) {
  var folderInfo = promptForTwoItems("Group Name (required)", "Description/Comments (optional)");
  if (!folderInfo)
    return; // user cancelled

  // Build the folder resource...
  // Folder Name
  var folderRes = passwordMakerHelper.rdfService.GetAnonymousResource();
  _updateDatasourceTarget(folderRes, folderNameURI, folderInfo.folderName);

  // Folder Description
  _updateDatasourceTarget(folderRes, folderDescURI, folderInfo.folderDescription);

  // Make the (empty) sequence, and append it to the container (e.g., the accounts sequence)
  var newContainer = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, folderRes);
  containerSeq.AppendElement(folderRes);
  
  passwordMakerHelper.datasourceFlusher.Flush();
}

function onCreateNewFolder() {
  createNewFolder("name", "description", passwordMakerHelper.accountsContainer);
}

/**
 * Called to create a new account in the currently
 * selected folder.
 */
function onCreateNewAccount() {
  var folderRes = getSelectedItemAsResource(accountsTree);
  if (!folderRes)
    return;
  var containerRes = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, folderRes);

  var defaultCells = getTreeCells(accountsTree, 0);
  var accountRes = passwordMakerHelper.rdfService.GetAnonymousResource();
  // Define the resources
  _updateDatasourceTarget(accountRes, "name", "New Account");
  _updateDatasourceTarget(accountRes, "description", "None");
  _updateDatasourceTarget(accountRes, "whereLeetLB", defaultCells[2].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "leetLevelLB", defaultCells[3].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "hashAlgorithmLB", defaultCells[4].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "passwordLength", defaultCells[5].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "urlToMatch", defaultCells[10].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "urlToUse", defaultCells[11].getAttribute("label"));  
  _updateDatasourceTarget(accountRes, "usernameTB", defaultCells[12].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "counter", defaultCells[13].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "charset", defaultCells[14].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "prefix", defaultCells[15].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "suffix", defaultCells[16].getAttribute("label"));
  _updateDatasourceTarget(accountRes, "autoPopulate", defaultCells[17].getAttribute("label"));

  // Add the resource to the container (folder)
  containerRes.AppendElement(accountRes);
  passwordMakerHelper.datasourceFlusher.Flush();

  // Open the selected item (the parent folder of the new accounts we just created)
  // if it's not already open.
  if (!accountsTree.view.isContainerOpen(accountsTree.currentIndex))
    accountsTree.view.toggleOpenState(accountsTree.currentIndex);

  passwordMaker.loadURLs(); // refresh arrays
  // Select the last item in the now-open folder (the newly-created account)
  var newAccountId = document.getElementById(accountRes.Value);
  var idx = accountsTree.view.getIndexOfItem(newAccountId);
  if (idx >= 0) {
    accountsTree.view.selection.select(idx); 
    // Open the account-settings dialog for the new item
    accountSettingsCmd.doCommand();
  }
}

/**
 * Called to update a single target.
 */
function _updateDatasourceTarget(subjectRes, predicateStr, literalStr) {
  var newTargetRes = passwordMakerHelper.rdfService.GetLiteral(literalStr);
  var predicateRes = passwordMakerHelper.rdfService.GetResource("http://passwordmaker.mozdev.org/rdf#" + predicateStr);
  try {
    var oldTargetRes = passwordMakerHelper.datasource.GetTarget(subjectRes, predicateRes, true);
    if (oldTargetRes == null) {
      passwordMakerHelper.datasource.Assert(subjectRes, predicateRes, newTargetRes, true);
    }
    else {
      passwordMakerHelper.datasource.Change(subjectRes, predicateRes, oldTargetRes, newTargetRes);
    }
  }
  catch (e) {
    dump(predicateStr + "\n");
    dump(e + "\n");
  }
}

/**
 * Return the RDF resource associated with the
 * selected tree item.
 */
function getSelectedItemAsResource(tree) {
  // Get id of selected item in tree
  if (tree.currentIndex > -1) {
    var subjectId = tree.builderView.getItemAtIndex(tree.currentIndex).getAttribute("id"); // e.g., rdf:#$JZPes
    return passwordMakerHelper.rdfService.GetResource(subjectId);
  }
  return null;
}

function deleteNodes(subject) {
  var isContainer = passwordMakerHelper.rdfContainerUtils.IsContainer(passwordMakerHelper.datasource, subject);
  if (isContainer) {
    // Subject is a container (folder)
    var container = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, subject);
    var children = container.GetElements();
    while (children.hasMoreElements()){
      var child = children.getNext();
      if (child instanceof Components.interfaces.nsIRDFResource){
        container.RemoveElement(child, true);
        children = container.GetElements();
        prune(child);
        // Delete fields container and elements, if they exist
        child = passwordMakerHelper.rdfService.GetResource(child.Value + "/fields");
        deleteNodes(child);
      }
    }
  }
  else {
    // Not a container (folder). Remove reference
    // to this subject in whichever container contains it.
    var arcsIn = passwordMakerHelper.datasource.ArcLabelsIn(subject);
    while (arcsIn.hasMoreElements()) {
      var arc = arcsIn.getNext();
      if (arc instanceof Components.interfaces.nsIRDFResource) {
        if (passwordMakerHelper.rdfContainerUtils.IsOrdinalProperty(arc)) {
          var parentRes = passwordMakerHelper.datasource.GetSource(arc, subject, true);
          var parentResContainer = passwordMakerHelper.rdfContainerUtils.MakeSeq(passwordMakerHelper.datasource, parentRes);
          parentResContainer.RemoveElement(subject, true);
        }
      }
    }
  }
  prune(subject);
  passwordMakerHelper.datasourceFlusher.Flush();
}

function prune(subject) {
  // Remove all remaining tuples
  var targets = passwordMakerHelper.datasource.ArcLabelsOut(subject);
  while (targets.hasMoreElements()){
    var predicate = targets.getNext();
    if (predicate instanceof Components.interfaces.nsIRDFResource){
      var target = passwordMakerHelper.datasource.GetTarget(subject, predicate, true);
      passwordMakerHelper.datasource.Unassert(subject, predicate, target);
    }
  }
}

function deleteTreeItem(tree, container) {
  var subject = getSelectedItemAsResource(tree);
  if (!subject)
    return false;

  var isContainer = passwordMakerHelper.rdfContainerUtils.IsContainer(passwordMakerHelper.datasource, subject);
  if (!passwordMakerHelper.ask(this, isContainer?"Are you sure you want to delete this item and all items below it?":"Are you sure you want to delete this item?"))
    return false;

  deleteNodes(subject);
  container.RemoveElement(subject, true); // remove from the container
  passwordMakerHelper.datasourceFlusher.Flush();
  return true;
}

function onDeleteTreeItem() {
  var subject = getSelectedItemAsResource(accountsTree);
  if (!subject)
    return;
  if (deleteTreeItem(accountsTree, passwordMakerHelper.accountsContainer)) {
    // Delete fields container and elements, if they exist
    subject = subject.QueryInterface(Components.interfaces.nsIRDFResource);
    subject = passwordMakerHelper.rdfService.GetResource(subject.Value + "/fields");
    deleteNodes(subject);
    passwordMakerHelper.datasourceFlusher.Flush();
    passwordMaker.loadURLs(); // refresh arrays   
  }
}

function getUrlToUse() {
    var cells = getTreeCells(accountsTree);
    return accountsTree.currentIndex == 0 ?
      
      // Default Options
      passwordMaker.calculateURI(cells[6].getAttribute("label") == "true", // protocolCB
        cells[7].getAttribute("label") == "true", // subdomainCB
        cells[8].getAttribute("label") == "true", // domainCB
        cells[9].getAttribute("label") == "true", // pathCB
        cells[9].getAttribute("label") == "true") : // pathCB
      
      // A specific account
      cells[11].getAttribute("label");
}

/*
  Returns the following array...

  0 : <treecell field="accountName" label="rdf:http://passwordmaker.mozdev.org/rdf#name"/>
  1 : <treecell field="accountDescription" label="rdf:http://passwordmaker.mozdev.org/rdf#description"/>
  2 : <treecell field="whereLeetLB" label="rdf:http://passwordmaker.mozdev.org/rdf#whereLeetLB"/>
  3 : <treecell field="leetLevelLB" label="rdf:http://passwordmaker.mozdev.org/rdf#leetLevelLB"/>
  4 : <treecell field="hashAlgorithmLB" label="rdf:http://passwordmaker.mozdev.org/rdf#hashAlgorithmLB"/>
  5 : <treecell field="passwordLength" label="rdf:http://passwordmaker.mozdev.org/rdf#passwordLength"/>
  6 : <treecell field="protocolCB" label="rdf:http://passwordmaker.mozdev.org/rdf#protocolCB"/>
  7 : <treecell field="subdomainCB" label="rdf:http://passwordmaker.mozdev.org/rdf#subdomainCB"/>
  8 : <treecell field="domainCB" label="rdf:http://passwordmaker.mozdev.org/rdf#domainCB"/>
  9 : <treecell field="pathCB" label="rdf:http://passwordmaker.mozdev.org/rdf#pathCB"/>
  10 : <treecell field="urlToMatch" label="rdf:http://passwordmaker.mozdev.org/rdf#urlToMatch"/>
  11 : <treecell field="urlToUse" label="rdf:http://passwordmaker.mozdev.org/rdf#urlToUse"/>
  12 : <treecell field="usernameTB" label="rdf:http://passwordmaker.mozdev.org/rdf#usernameTB"/>
  13 : <treecell field="counter" label="rdf:http://passwordmaker.mozdev.org/rdf#counter"/>
  14 : <treecell field="charset" label="rdf:http://passwordmaker.mozdev.org/rdf#charset"/>
*/
function getTreeCells(tree, index) {
  if (index == null)
    index = tree.currentIndex;
  var selectedTreeItem = tree.view.getItemAtIndex(index);   // <treeitem/>
  var selectedTreeRow = selectedTreeItem.firstChild;   // <treerow/>
  return selectedTreeRow.childNodes;  // <treecell/> NodeList
}

function onOtherFields() {
  var subjectRes = getSelectedItemAsResource(accountsTree);
  if (!subjectRes)
    return;
  var params = {inn:{passwordMakerHelper:passwordMakerHelper, passwordMaker:passwordMaker, accountRes:subjectRes}, out : null};
  window.openDialog("chrome://passwdmaker/chrome/fields.xul", "", "chrome, dialog, modal, resizable=yes", params).focus();
}

function onAccountSettings() {
  /*if (PasswordMakerHelper._masterPassword) {
    var mpw = passwordMakerHelper.getDecryptedMasterPassword();
    if (!mpw || mpw != passwordMaster.value) {
      return;
    }
  }*/
  var subjectRes = getSelectedItemAsResource(accountsTree);
  if (!subjectRes)
    return;

  var isDefaultsAccount = accountsTree.currentIndex == 0;
  var isFolder = accountsTree.view.getItemAtIndex(accountsTree.currentIndex).getAttribute("container") == "true";

  // Get current settings from the UI (we could get them from the RDF instead)
  var cells = getTreeCells(accountsTree);
  var accountSettings = {isFolder : isFolder, isDefaultsAccount : isDefaultsAccount,
    accountName : cells[0].getAttribute("label"), 
    accountDescription : cells[1].getAttribute("label"), whereLeetLB : cells[2].getAttribute("label"),
    leetLevelLB : cells[3].getAttribute("label"), hashAlgorithmLB : cells[4].getAttribute("label"),
    passwordLength : cells[5].getAttribute("label"), protocolCB : cells[6].getAttribute("label"),
    subdomainCB : cells[7].getAttribute("label"), domainCB : cells[8].getAttribute("label"),
    pathCB : cells[9].getAttribute("label"), urlToMatch : (isDefaultsAccount ? getUrlToUse() : cells[10].getAttribute("label")),
    urlToUse : (isDefaultsAccount ? getUrlToUse() : cells[11].getAttribute("label")), usernameTB : cells[12].getAttribute("label"),
    counter : cells[13].getAttribute("label"), charset : cells[14].getAttribute("label"), prefix : cells[15].getAttribute("label"),
    suffix : cells[16].getAttribute("label"), autoPop : cells[17].getAttribute("label")};

  var params = {inn:{accountSettings : accountSettings, passwordMakerHelper : passwordMakerHelper,
    passwordMaster : passwordMaster.value, passwordMaker : passwordMaker, accountRes : subjectRes}, out : null};

  window.openDialog("chrome://passwdmaker/chrome/accountsettings.xul", "",
    "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK - update datasource
    accountSettings = params.out.accountSettings;
    _updateDatasourceTarget(subjectRes, "name", accountSettings.accountName);
    _updateDatasourceTarget(subjectRes, "description", accountSettings.accountDescription);
    if (!isFolder) {
      // This isn't a folder -- continue saving the other attributes
      _updateDatasourceTarget(subjectRes, "whereLeetLB", accountSettings.whereLeetLB);
      _updateDatasourceTarget(subjectRes, "leetLevelLB", accountSettings.leetLevelLB);
      _updateDatasourceTarget(subjectRes, "hashAlgorithmLB", accountSettings.hashAlgorithmLB);
      _updateDatasourceTarget(subjectRes, "passwordLength", accountSettings.passwordLength);
      if (!isDefaultsAccount) {
        // Especially don't write urlToMatch, otherwise the Default will match weird stuff!
        _updateDatasourceTarget(subjectRes, "urlToMatch", accountSettings.urlToMatch); 
        _updateDatasourceTarget(subjectRes, "urlToUse", accountSettings.urlToUse);
      }
      _updateDatasourceTarget(subjectRes, "usernameTB", accountSettings.usernameTB);
      _updateDatasourceTarget(subjectRes, "counter", accountSettings.counter);
      _updateDatasourceTarget(subjectRes, "charset", accountSettings.charset);
      _updateDatasourceTarget(subjectRes, "prefix", accountSettings.prefix);
      _updateDatasourceTarget(subjectRes, "suffix", accountSettings.suffix);
      _updateDatasourceTarget(subjectRes, "autoPopulate", accountSettings.autoPop ? "true" : "false");
      if (isDefaultsAccount) {
        // The defaults node is selected
        _updateDatasourceTarget(subjectRes, "protocolCB", accountSettings.protocolCB ? "true" : "false");
        _updateDatasourceTarget(subjectRes, "subdomainCB", accountSettings.subdomainCB ? "true" : "false");
        _updateDatasourceTarget(subjectRes, "domainCB", accountSettings.domainCB ? "true" : "false");
        _updateDatasourceTarget(subjectRes, "pathCB", accountSettings.pathCB ? "true" : "false");
      }
    }
    params = null; // proactive clearing of sensitive data
    passwordMakerHelper.datasourceFlusher.Flush(); // gen password AFTER flushing so tree is up-to-date for generation
    if (!isFolder) {
      updateURLLabel();
      generatepassword();
    }
    passwordMaker.loadURLs(); // refresh arrays
  }
}

