/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

// References to the UI elements
var clearClipboardCB, clearClipboard, maskGeneratedPasswordCB, pwShortcutLB, hideMPWFieldCB, mpwDeck,
  confirmMPWCB, viewPwdFieldsCB, removePwdAutoCompleteCB;

function initGlobalSettingsPanel() {

  // Get references to the UI elements
  clearClipboardCB = document.getElementById("clearClipboardCB");
  clearClipboard = document.getElementById("clearClipboard");
  maskGeneratedPasswordCB = document.getElementById("maskGeneratedPasswordCB");
  pwShortcutLB = document.getElementById("pwShortcutLB");
  hideMPWFieldCB = document.getElementById("hideMPWFieldCB");
  mpwDeck = document.getElementById("mpwDeck");
  confirmMPWCB = document.getElementById("confirmMPWCB");
  viewPwdFieldsCB = document.getElementById("viewPwdFieldsCB");
  removePwdAutoCompleteCB = document.getElementById("removePwdAutoCompleteCB");
  
  // Masking
  maskGeneratedPasswordCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.maskMasterPasswordPredicateRes) == "true";
  
  // Auto-clear clipboard
  clearClipboardCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardPredicateRes) == "true";

  // Auto-clear clipboard secs
  clearClipboard.value = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardSecondsPredicateRes);

  // Password Shortcut (ALT-`)
  pwShortcutLB.value = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.passwordShortcutPredicateRes);
  hideMPWFieldCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.hideMasterPasswordFieldPredicateRes) == "true";
  if (passwordMaster.value == "") {
    // Can't ever be checked if the mpw is empty
    hideMPWFieldCB.checked = false;
    passwordMakerHelper.saveGlobalSetting("hideMasterPasswordField", "false");
  }

  if (!hideMPWFieldCB.checked)
    mpwDeck.selectedIndex = 1;
  
  confirmMPWCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.confirmMPWPredicateRes) == "true";
  viewPwdFieldsCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.viewPwdFieldsPredicateRes) == "true";
  removePwdAutoCompleteCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.removePwdAutoCompletePredicateRes) == "true";

  onClearClipboardCB();
}

function onConfirmMPW() {
  passwordMakerHelper.saveGlobalSetting("confirmMPW", confirmMPWCB.checked ? "true":"false");
}

function onViewPwdFields() {
  passwordMakerHelper.saveGlobalSetting("viewPwdFields", viewPwdFieldsCB.checked ? "true":"false");
  togglePasswordField();
}

function onRemovePwdAutoComplete() {
  passwordMakerHelper.saveGlobalSetting("removePwdAutoComplete", removePwdAutoCompleteCB.checked ? "true":"false");
  toggleAutoComplete();
}

function toggleAutoComplete() {
    var inputElements = _getInputFields(); 
    for (var i=0; i<inputElements.length; i++)
      inputElements[i].setAttribute("autocomplete", removePwdAutoCompleteCB.checked ? "on" : "off");
}

function _getInputFields() {
  // Get recent wnd
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
  var recentWindow = wm.getMostRecentWindow("navigator:browser");
  if (recentWindow)
    return recentWindow.content.document.getElementsByTagName("input");
  else
    return new Array();
}

/**
 * Similar to passwordMaker.modifyPasswordFieldAttributes(), but that
 * version is optimized for speed since it runs every pageLoad().
 */
var werePasswordFields = new Array();
function togglePasswordField() {
  if (viewPwdFieldsCB.checked) { 
    var inputElements = _getInputFields();      
    werePasswordFields = new Array();         
    for (var i=0; i<inputElements.length; i++) {
      if (inputElements[i].type == "password") { // .type is always lower-case  
        inputElements[i].type = "edit";
        werePasswordFields[werePasswordFields.length] = inputElements[i];
      }
    }
  }
  else {
    for (var i=0; i<werePasswordFields.length; i++)
      werePasswordFields[i].type = "password";
    werePasswordFields = new Array(); 
  }
}

function onHideMPWField(fromBtn) {
  if (fromBtn) {
    // User wants to turn off masking
    showMPWField(); 
  }
  else {
    if (hideMPWFieldCB.checked) {
      if (passwordMaster.value == "") {
        passwordMakerHelper.prompts.alert(this, "PasswordMaker", "You must first enter a master password");
        hideMPWFieldCB.checked = false;
      }
      else
        mpwDeck.selectedIndex = 0;
    }
    else
      showMPWField();
  }
  passwordMakerHelper.saveGlobalSetting("hideMasterPasswordField", hideMPWFieldCB.checked ? "true":"false");
}

function showMPWField() {
  // User wants to turn off masking
  // Firefox unchecked the checbkox, even though the user
  // hasn't entered the master pw yet. Check it here so
  // the UI matches the current state of affairs.
  hideMPWFieldCB.checked = true;
  if (passwordMakerHelper.prompwMPW(passwordMaster.value)) {
    hideMPWFieldCB.checked = false;
    mpwDeck.selectedIndex = 1;
  }
}

/**
 * Method which fires when the Clear Clipboard checkbox
 * has been clicked.
 */
function onClearClipboardCB() {
  clearClipboard.disabled = !clearClipboardCB.checked;
  if (clearClipboardCB.checked && clearClipboard.value == "")
    clearClipboard.value = "10";
  passwordMakerHelper.saveGlobalSetting("autoClearClipboard", clearClipboardCB.checked ? "true":"false");
}

function onClearClipboard() {
  if (/\D/.test(clearClipboard.value))
    clearClipboard.value="10";
  passwordMakerHelper.saveGlobalSetting("autoClearClipboardSeconds", clearClipboard.value);
}

function onPWShortcutLBChanged() {
  passwordMakerHelper.saveGlobalSetting("passwordShortcut", pwShortcutLB.value);
  passwordMaker.passwordShortcut = pwShortcutLB.value;
}

function onMaskGeneratedPasswordButton() {
  if (!maskGeneratedPasswordCB.checked) {
    // User wants to turn off masking
    if (passwordMaster.value != "") {
      // A master password has been set, so we can't allow
      // them to turn off masking just yet
      // Firefox unchecked the checbkox, even though the user
      // hasn't entered the master pw yet. Check it here so
      // the UI matches the current state of affairs.
      maskGeneratedPasswordCB.checked = true;
      // Prompt user for the master password before turning off
      // generated password masking
      if (passwordMakerHelper.prompwMPW(passwordMaster.value))
        maskGeneratedPasswordCB.checked = false;
    }
  }
  passwordMakerHelper.saveGlobalSetting("maskMasterPassword", maskGeneratedPasswordCB.checked ? "true":"false");
  passwordGenerated.setAttribute("type", maskGeneratedPasswordCB.checked ? "password" : "");
}